
use crate::model::{
	events::EventsCollection,
	ical_bridge::load_calendar,
};
use crate::ui::{
	controls::Controls,
	calendar::{
		CalendarParams,
		CalendarView,
		CalendarViewMode as ViewMode,
	}
};
#[cfg(feature = "tracing")]
use crate::tracepoints;
use chrono::{Datelike, NaiveDate, Months, Utc, Days};
use std::path;
use iced::{
    Alignment,
    Command,
    Element,
    Length,
    widget::{
	    Column,
	    Container,
	},
};

#[derive(Debug, Clone, Copy)]
pub enum Message {
    NextWeek,
    PrevWeek,
    NextMonth,
    PrevMonth,
    NextYear,
    PrevYear,
    ViewModeSelected(ViewMode),
}

#[derive(Default)]
pub struct CalendarApp {
    view_date: NaiveDate,
    controls: Controls,
    events: EventsCollection,
}

impl std::fmt::Display for ViewMode {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match self {
                ViewMode::Week => "Week",
                ViewMode::Month => "Month",
                ViewMode::Year => "Year",
            }
        )
    }
}

impl CalendarApp {
	pub fn with_files(calendar_paths: std::vec::Vec<path::PathBuf>) -> Self {
        let view_date = Utc::now().date_naive();
        let mut app = CalendarApp {
            view_date,
            events: EventsCollection::new(),
            ..CalendarApp::default()
        };
        for calendar_path in calendar_paths {
            load_calendar(&calendar_path, &mut app.events);
        }
        app
	}

    pub fn update(&mut self, message: Message) -> Command<Message> {
        match message {
            Message::PrevWeek => {
                self.view_date = self.view_date - Days::new(7);
            }
            Message::NextWeek => {
                self.view_date = self.view_date + Days::new(7);
            }
            Message::PrevMonth => {
                self.view_date = self.view_date - Months::new(1);
            }
            Message::NextMonth => {
                self.view_date = self.view_date + Months::new(1);
            }
            Message::PrevYear => {
                self.view_date = self.view_date - Months::new(12);
            }
            Message::NextYear => {
                self.view_date = self.view_date + Months::new(12);
            }
            Message::ViewModeSelected(mode) => {
                self.controls.mode = Some(mode);
            }
        }
        Command::none()
    }

    pub fn view(&self) -> Element<Message> {
        #[cfg(feature = "tracing")]
        tracepoints::calendar::view_entry();
        let content = Column::new()
            .align_items(Alignment::Center)
            .push(self.controls.view(self.view_date, self.view_date.year()))
            .push(CalendarView::new(self.controls.mode.unwrap_or(ViewMode::Year), &CalendarParams::new(), self.view_date, &self.events))
            ;

        let container = Container::new(content)
            .width(Length::Fill)
            .height(Length::Fill)
            .center_x(Length::Fill)
            .center_y(Length::Fill)
            .into();

        #[cfg(feature = "tracing")]
        tracepoints::calendar::view_exit();

        container
    }
}
